/** AgentHT00000X.java
 * -*- java -*-
 *
 * RCS: $Id: AgentU000000X.java,v 1.1 2006/01/28 04:24:02 kanmy Exp kanmy $
 *
 * RCS: $Log: AgentU000000X.java,v $
 * RCS: Revision 1.1  2006/01/28 04:24:02  kanmy
 * RCS: Initial revision
 * RCS:
 */
import java.io.*;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

public class AgentU000000X {

    static final String identify = "Min\'s breakthrough agent v1.0 - I honk for game players";
    static final String email = "u000000@nus.edu.sg";

    static final int MOVE = 1;	// modes of operation
    static final int IDENTIFY = 2;
    static final int UNKNOWN = -1;

    static final int URGENT_TIME = 2800; // when do we consider we are out of time?
    static final String URGENT_MOVE = "3 3"; // the default move if we run out of time

    static final int DEBUG = 1;
    static final int NO_DEBUG = 2;

    static final int FORWARD = 0;
    static final int DIAG_LEFT = 1;
    static final int DIAG_RIGHT = 2;
    
    char[][] board = null;	// the board as an array of characters
    int rows = 6;		// the dimension of the board, will get overridden by input
    int cols = 6;
    int debug = NO_DEBUG;	// turn this on for debugging
    int turnNumber = -1;	// which turn number

    char mySymbol = 'w';	// who am I playing?
    char empty = '-';
    char opSymbol = 'b';

    /**
     * class that implements a psuedo interrupt.  Wake after URGENT_TIME milliseconds, 
     * more or less (no real-time guarantees)
     */
    static class UrgentMove extends TimerTask {
	public void run() {
	    System.out.println (URGENT_MOVE); // don't think; just move

	    System.exit(0);	// exit with normal status
	}
    } // end UrgentMove

    /** 
     * class that implements a pair of integer coordinates
     */
    class PairCoord {
	int sx;
	int sy;
	int ex;
	int ey;

	/**
	 * constructor for a Pair of coordinates
	 */
	PairCoord (int sx, int sy, int ex, int ey) {
	    this.sx = sx;
	    this.sy = sy;
	    this.ex = ex;
	    this.ey = ey;
	}

	/** accessor methods */
	int getSX () { return this.sx; }
	int getSY () { return this.sy; }
	int getEX () { return this.ex; }
	int getEY () { return this.ey; }
	
	/** mutation methods */
	void setSX (int sx) { this.sx = sx; }
	void setSY (int sy) { this.sy = sy; }
	void setEX (int ex) { this.ex = ex; }
	void setEY (int ey) { this.ey = ey; }

	/** takes a pair of x,y coordinates, converts to standard board notation */
	String encode (int x, int y) {
	    return ("" + new Character((char) ('A'+x)) + (y+1));
	}
    }

    /**
     * Method to print the board, for debugging
     */
    void printBoard () {
	int x;
	int y;
	for (y = (this.rows-1); y >= 0; y--) {
	    System.out.print((y+1) + "\t"); // print y axis
	    for (x = 0; x < this.cols; x++) {
		System.out.print (board[x][y] + " "); // print board spaces
	    } 
	    System.out.println();
	} 
	System.out.print("\n\t");
	for (x = 0; x < this.cols; x++) { // print x axis
	    System.out.print(new Character((char) ('A'+x)) + " ");
	}
	System.out.println();
    }

    /**
     * Method to make a move.  game state is stored in the class variables
     * 
     *
     * Method for you to implement. You want to make your modifications here.
     */
    PairCoord findMove () {
	Random r = new Random();
	int sx = -1;
	int sy = -1;
	int ex = -1;
	int ey = -1;
	int move = -1;
	boolean valid = false;

	if (this.debug == DEBUG) {
	    printBoard();
	}

	// get a random values for x and y as long as board[x][y] 
	try {
	    while (!valid) {
		do {		// get a starting coordinate
		    sx = r.nextInt(this.cols);
		    sy = r.nextInt(this.rows);
		} while (sx == -1 || sy == -1 || board[sx][sy] != mySymbol);

		move = r.nextInt(3);

		switch (move) {	// pick from three possible moves and check if valid
		case FORWARD:		// move forward
		    if (sy != this.rows - 1 && board[sx][sy+1] == empty) {
			ex = sx;
			ey = sy+1;
			valid = true;
		    }
		    break;
		case DIAG_LEFT:
		    if (sy != this.rows - 1 && sx != 0 && board[sx-1][sy+1] != mySymbol) {
			ex = sx-1;
			ey = sy+1;
			valid = true;
		    }
		    break;
		case DIAG_RIGHT:
		    if (sy != this.rows - 1 && sx != this.cols - 1 && board[sx+1][sy+1] != mySymbol) {
			ex = sx+1;
			ey = sy+1;
			valid = true;
		    }
		    break;
		}
	    }
	} catch (Exception e) { ; }

	PairCoord action = new PairCoord(sx,sy,ex,ey); 
	return (action);
    } // end of findMove

    /**
     * Method to read the input and store into appropriate data structures
     */
    int readInput () {
	// temporary variables
	String line;		
	String[] tokens;
	int lineNum = 0;

	int mode = UNKNOWN;	// mode of operation

	// Read from standard input
	try {
	    BufferedReader in = new BufferedReader(new InputStreamReader (System.in));
	    while ((line = in.readLine()) != null) {
		lineNum++;
		// decide which mode of operation we are in depending on the first line of input
		if (lineNum == 1) {
		    if (line.equals("identify")) { 
			mode = IDENTIFY;
		    } else if (line.equals("move")) {
			mode = MOVE;
		    } else {
			throw new Exception ("Unknown mode of operation!");
		    }
		}
		
		// read data from standard input into data structures for manipulation later
		if (mode == MOVE) {
		    switch (lineNum) {
		    case 1: break; // skip "move" line
		    case 2:     // turn number
			turnNumber = Integer.parseInt(line);
			break;
		    case 3: 	// size of board
			tokens = line.split(" ");
			cols = Integer.parseInt(tokens[0]);
			rows = Integer.parseInt(tokens[1]);
			board = new char[cols][rows]; // get memory for the board
			break;
		    default:	// all board lines
			int y = rows - (lineNum - 3); // 4th line starts board, 0,0 is lower left
			for (int x = 0; x < line.length(); x++) {
			    board[x][y] = line.charAt(x);
			}
		    }
		} 
	    }
	} catch (Exception e) {	// oops, something bad happened.  print the stack trace and exit.
	    e.printStackTrace();
	    System.exit(-1);
	}
	return mode;
    } // end of readInput

    /**
     * The main class to the work
     */
    public static void main (String[] args) {
	AgentU000000X agent = new AgentU000000X();

	Timer timer = new Timer();
	// timer.schedule (new UrgentMove(), URGENT_TIME);

	int mode = agent.readInput();

	switch (mode) {
	case IDENTIFY:
	    System.out.println(identify); 
	    break;
	case MOVE: 
	    PairCoord move = agent.findMove (); // get our move
	    System.out.println (move.encode(move.getSX(),move.getSY()) + move.encode(move.getEX(),move.getEY()));
	    timer.cancel();
	    break;
	}
	System.exit(0);
    } // end of main
}
